@@ -37,18 +37,12 @@ BayonneBinder("ivrscript1")
 			(Check)&Checks::chkIgnore},
                 {"cdr", false, (Method)&ScriptMethods::scrNop,   
                         (Check)&Checks::chkCDR},   
-                {"select", false, (Method)&ScriptMethods::scrNop,  
-                        (Check)&Checks::chkSelect}, 
-                {"incoming", false, (Method)&ScriptMethods::scrNop,
-                        (Check)&Checks::chkAssign},
-                {"caller", false, (Method)&ScriptMethods::scrNop,   
-                        (Check)&Checks::chkAssign},  
-                {"assign", false, (Method)&ScriptMethods::scrNop,
-                        (Check)&Checks::chkAssign},    
-                {"register", false, (Method)&ScriptMethods::scrNop,
-                        (Check)&Checks::chkRegister}, 
+                {"_keydata_", false, (Method)&Methods::scrKeydata,
+                        (Check)&Checks::chkKeydata}, 
                 {"key", true, (Method)&Methods::scrKey,
                         (Check)&Checks::chkKey},  
+                {"select", false, (Method)&ScriptMethods::scrNop,  
+                        (Check)&Checks::chkSelect}, 
 		{"startxml", false, (Method)&Methods::scrStart,
 			(Check)&Checks::chkStart},
 		{"parsexml", false, (Method)&Methods::scrParse,
@@ -72,7 +66,7 @@ BayonneBinder("ivrscript1")
 	PersistProperty::load();
 
 	addConfig("timeslot.conf");
-	addConfig("caller.conf");
+	addConfig("connect.conf");
 	addConfig("dialed.conf");
 	addConfig("runlevel.conf");
 
@@ -83,6 +77,46 @@ BayonneBinder("ivrscript1")
 	allocateLocal();
 };
 
+Script::Name *Binder::getIncoming(ScriptImage *img, BayonneSession *s, Event *event)
+{
+    const char *did;
+    char buf[80];
+    Name *scr = NULL;
+
+    if(sla[0])
+    {
+        scr = img->getScript(sla);
+        if(scr)
+            return scr;
+    }
+
+    did = s->getSymbol("session.hostname");
+    if(did && !*did)
+        did = NULL;
+
+    if(did)
+    {
+        snprintf(buf, sizeof(buf), "dialed::%s", did);
+        scr = img->getScript(buf);
+        if(scr)
+            return scr;
+    }
+
+	if(event->start.scr)
+		return event->start.scr;
+
+	did = getRegistryId(s->getSymbol("session.dialed"));
+    if(did)
+    {
+        snprintf(buf, sizeof(buf), "dialed::%s", did);
+        scr = img->getScript(buf);
+        if(scr)
+            return scr;
+    }
+	
+    return BayonneBinder::getIncoming(img, s, event);
+}
+
 bool Binder::select(ScriptInterp *interp)
 {
         char buf[65]; 
