@@ -15,6 +15,11 @@
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
 #include "module.h"
+#ifndef	WIN32
+#include "private.h"
+#else
+#define	CONFIG_FILES	"C:\\Program Files\\GNU Telephony\\Bayonne Config"
+#endif
 
 namespace binder {
 using namespace ost;
@@ -26,6 +31,12 @@ Binder::Binder() :
 BayonneBinder("ivrscript")
 {
         static ScriptInterp::Define interp[] = { 
+                {"endinput", false, (Method)&Methods::scrEndinput,
+                        (Check)&Checks::chkEndinput}, 
+		{"endform", false, (Method)&Methods::scrEndform,
+			(Check)&Checks::chkConditional},
+		{"form", false, (Method)&Methods::scrForm,
+			(Check)&Checks::chkIgnore},
                 {"cdr", false, (Method)&ScriptMethods::scrNop,   
                         (Check)&Checks::chkCDR},   
                 {"dial", false, (Method)&Methods::scrDial,
@@ -38,69 +49,131 @@ BayonneBinder("ivrscript")
                         (Check)&Checks::chkAssign},  
                 {"assign", false, (Method)&ScriptMethods::scrNop,
                         (Check)&Checks::chkAssign},    
+                {"_keydata_", false, (Method)&Methods::scrKeydata,
+                        (Check)&Checks::chkKeydata},
                 {"register", false, (Method)&ScriptMethods::scrNop,
+                        (Check)&Checks::chkRegister},
+                {"extern", false, (Method)&ScriptMethods::scrNop,
                         (Check)&Checks::chkRegister}, 
+		{"extension", false, (Method)&ScriptMethods::scrNop,
+			(Check)&Checks::chkRegister},
+                {"gateway", false, (Method)&ScriptMethods::scrNop,
+                        (Check)&Checks::chkRegister},
                 {"key", true, (Method)&Methods::scrKey,
                         (Check)&Checks::chkKey},  
-                {"start", false, (Method)&Methods::scrStart,
-                        (Check)&Checks::chkStart},  
-                {"connect", false, (Method)&Methods::scrStart,
-                        (Check)&Checks::chkConnect},  
-                {"busysignal", false, (Method)&Methods::scrSignal,
-                        (Check)&Checks::chkSignal},
-                {"failsignal", false, (Method)&Methods::scrSignal,
-                        (Check)&Checks::chkSignal}, 
-                {"detach", true, (Method)&Methods::scrDetach,
-                        (Check)&Checks::chkCancel},
-                {"cancel", true, (Method)&Methods::scrCancel,       
-                        (Check)&Checks::chkCancel}, 
-                {"join", false, (Method)&Methods::scrJoin,      
-                        (Check)&Checks::chkJoin},
-                {"wait", false, (Method)&Methods::scrWait,
-                        (Check)&Checks::chkWait}, 
+		{"stop", false, (Method)&Methods::scrStop,
+			(Check)&Checks::chkStop},
+                {"cancel", false, (Method)&Methods::scrStop,
+                        (Check)&Checks::chkStop},
+		{"start", false, (Method)&Methods::scrStart,
+			(Check)&Checks::chkStart},
+		{"connect", false, (Method)&Methods::scrConnect,
+			(Check)&Checks::chkConnect},
+		{"join", false, (Method)&Methods::scrJoin,
+			(Check)&Checks::chkJoin},
+		{"recall", false, (Method)&Methods::scrRecall,
+			(Check)&Checks::chkRecall},
+		{"redial", false, (Method)&Methods::scrRedial,
+			(Check)&Checks::chkRedial},
+        {"reroute", false, (Method)&Methods::scrRedial,
+            (Check)&Checks::chkRedial},
+		{"feature", false, (Method)&Methods::scrFeature,
+			(Check)&Checks::chkFeature},
+		{"release", true, (Method)&Methods::scrRelease,
+			(Check)&Checks::chkRecall},
                 {NULL, false, NULL, NULL}}; 
 
 	bind(interp);
 	slog.info("binding ivrscript...");
 	PersistProperty::load();
 
-	ScriptInterp::addConditional("key", &testKey); 
-	Bayonne::provision_ripple = false;
+	ScriptInterp::addConditional("key", &testKey);
+	ScriptInterp::addConditional("registered", &testRegistered);
+	ScriptInterp::addConditional("available", &testAvailable);
+	ScriptInterp::addConditional("extension", &testExternal);
+	ScriptInterp::addConditional("external", &testExternal);
+	ScriptInterp::addConditional("reachable", &testReachable);
+	ScriptInterp::addConditional("destination", &testDestination);
+
 	Bayonne::use_prefix = true;
 	Bayonne::use_macros = true;
-	Bayonne::use_merge = false;
+	Bayonne::use_merge = true;
+
+	addConfig("timeslot.conf");
+	addConfig("connect.conf");
+	addConfig("feature.conf");
+	addConfig("caller.conf");
+	addConfig("dialed.conf");
+	addConfig("local.conf");
+	addConfig("runlevel.conf");
 };
 
-Script::Name *Binder::service(ScriptImage *img, ScriptInterp *interp, const char *sid)
+Script::Name *Binder::getIncoming(ScriptImage *img, BayonneSession *s, Event *event)
 {
-	char buf[65];
-	BayonneDriver *driver = (session(interp))->getDriver();
-	Name *scr;
+	const char *cid, *did, *uid;
+	char buf[80];
+	Name *scr = NULL;
 
-	if(!sid)
+	if(sla[0])
 	{
-                snprintf(buf, sizeof(buf), "down::%s", driver->getName());
-                scr = img->getScript(buf); 
-		if(!scr)
-			scr = img->getScript("down");
-		return scr;
+		scr = img->getScript(sla);
+		if(scr)
+			return scr;
+	}
+	did = s->getSymbol("session.hostname");
+	if(did && !*did)
+		did = NULL;
+
+	if(did)
+	{
+		snprintf(buf, sizeof(buf), "dialed::%s", did);
+		scr = img->getScript(buf);
+		if(scr)		
+			return scr;
 	}
 
-	if(!*sid)
-		return NULL;
+	uid = getRegistryId(s->getSymbol("session.identity"));
+	cid = getRegistryId(s->getSymbol("session.caller"));
 
-	snprintf(buf, sizeof(buf), "%s::%s", interp->getExternal("session.interface"));
-	scr = img->getScript(buf);
-	if(!scr)
-		scr = img->getScript(sid);
+	if(cid && !uid)
+	{
+		snprintf(buf, sizeof(buf), "caller::%s", cid);
+		scr = img->getScript(buf);
+		if(scr)
+			return scr;
+	}
 
-	return scr;
+	if(uid)
+	{
+		snprintf(buf, sizeof(buf), "local::%s", uid);
+		scr = img->getScript(buf);
+		if(scr)
+			return scr;
+	}
+
+	did = getRegistryId(s->getSymbol("session.dialed"));
+	if(did && uid && !stricmp(did, uid))
+	{
+		scr = img->getScript("local::_self_");
+		if(scr)
+			return scr;
+	}
+
+	if(did)
+	{
+		snprintf(buf, sizeof(buf), "dialed::%s", did);
+		scr = img->getScript(buf);
+		if(scr)
+			return scr;
+	}
+
+	return BayonneBinder::getIncoming(img, s, event);
 }
 
+
 bool Binder::select(ScriptInterp *interp)
 {
         char buf[65]; 
-	BayonneSession *s = (BayonneSession *)interp;
 
 	const char *cinfo = interp->getSymbol("session.info");
 	const char *ctype = interp->getExternal("session.type");
@@ -221,16 +294,33 @@ void Binder::down(void)
 
 bool Binder::reload(ScriptCompiler *img)
 {
-	ScriptCommand *server;
-	const char *prefix;
-	unsigned count;
-	char **keys;
-	char *ext;
+	ScriptCommand *server = img->getCommand();
+	const char *prefix, *cp;
+	unsigned count = server->Keydata::getCount();
+	char **keys = new char *[count + 1];
+	char buffer[65];
+	const char *ext;
 
-	server = img->getCommand();
-	count = server->Keydata::getCount();	
-	keys = new char *[count + 1];
+#ifdef  SCRIPT_DEFINE_TOKENS
+        const char *const *list = server->getList("definitions");
 
+        img->compileDefinitions("site.def");
+        while(list && *list)
+        {
+               	img->compileDefinitions(*list);
+               	++list;
+        }
+	cp = server->getLast("location");
+	if(cp)
+	{
+		snprintf(buffer, sizeof(buffer), "macros-%s.def", cp);
+		img->compileDefinitions(buffer);
+	}
+	img->compileDefinitions("macros.def");
+
+#endif
+
+	
 	count = server->Keydata::getIndex(keys, count);
 	while(count)
 	{
@@ -242,16 +332,30 @@ bool Binder::reload(ScriptCompiler *img)
 		if(ext && strstr(Bayonne::exec_extensions, ext))
 			goto compile;
 
+		if(ext && !stricmp(ext, ".conf"))
+			goto conf;
+
 		if(!ext || stricmp(ext, ".scr"))
 			continue;
-
 compile:
 		++Bayonne::compile_count;
 		prefix = server->getLast(prefix);
-		img->compile(prefix);
+		if(canAccess(prefix))
+			img->compile(prefix);
+		continue;
+
+conf:
+		++Bayonne::compile_count;
+		prefix = server->getLast(prefix);
+		if(canAccess(prefix))
+			img->compile(prefix);
 	}
 
-	prefix = server->getLast("addons");
+#ifdef	WIN32
+	prefix = "C:\\Program Files\\GNU Telephony\\Bayonne IVRScript";
+#else
+	prefix = BAYONNE_FILES "/ivrscript";
+#endif
 	if(prefix)
 	{
 		if(isDir(prefix))
@@ -306,7 +410,8 @@ void Binder::compileDir(const char *prefix, ScriptCompiler *img)
 compile:  
                 ++Bayonne::compile_count;
                 setString(pp, len, file);
-                img->compile(path);
+		if(canAccess(path))
+	                img->compile(path);
         }
 }  
 
@@ -314,6 +419,105 @@ bool Binder::testKey(ScriptInterp *interp, const char *v)
 {
         return PersistProperty::test(v);
 }
-  
+
+bool Binder::testRegistered(ScriptInterp *inter, const char *v)
+{
+	BayonneDriver *d = NULL;
+	char vbuf[8];
+	char *p;
+
+	setString(vbuf, sizeof(vbuf), v);
+	p = strchr(v, ':');
+	if(p)
+	{
+		*p = 0;
+		v = (const char *)++p;
+		d = BayonneDriver::get(vbuf);
+	}
+	if(!d)
+		d = BayonneDriver::getProtocol();
+	if(!d)
+		return false;
+
+	return d->isRegistered(v);
+}
+
+bool Binder::testExternal(ScriptInterp *interp, const char *v)
+{
+	BayonneDriver *d = NULL;
+	char vbuf[8];
+	char *p;
+
+	setString(vbuf, sizeof(vbuf), v);
+	p = strchr(v, ':');
+	if(p)
+	{
+		*p = 0;
+		v = (const char *)++p;
+		d = BayonneDriver::get(vbuf);
+	}
+	if(!d)
+		d = BayonneDriver::getProtocol();
+	if(!d)
+		return false;
+
+	return d->isExternal(v);
+}
+
+bool Binder::testAvailable(ScriptInterp *inter, const char *v)
+{
+	BayonneDriver *d = NULL;
+	char vbuf[8];
+	char *p;
+
+	setString(vbuf, sizeof(vbuf), v);
+	p = strchr(v, ':');
+	if(p)
+	{
+		*p = 0;
+		v = (const char *)++p;
+		d = BayonneDriver::get(vbuf);
+	}
+	if(!d)
+		d = BayonneDriver::getProtocol();
+	if(!d)
+		return false;
+
+	return d->isAvailable(v);
+}  
+	
+bool Binder::testDestination(ScriptInterp *interp, const char *v)
+{
+	char name[80];
+	ScriptImage *img = interp->getImage();
+	snprintf(name, sizeof(name), "dialed::%s", v);
+	
+	if(img->getScript(name))
+		return true;
+
+	return false;
+}
+
+bool Binder::testReachable(ScriptInterp *inter, const char *v)
+{
+	BayonneDriver *d = NULL;
+	char vbuf[8];
+	char *p;
+
+	setString(vbuf, sizeof(vbuf), v);
+	p = strchr(v, ':');
+	if(p)
+	{
+		*p = 0;
+		v = (const char *)++p;
+		d = BayonneDriver::get(vbuf);
+	}
+	if(!d)
+		d = BayonneDriver::getProtocol();
+	if(!d)
+		return false;
+
+	return d->isReachable(v);
+}  
 
 } // end namespace
