@@ -262,6 +262,8 @@ private:
 #endif
 
 public:
+	static char dtmf_keymap[256];
+
 	static void snmptrap(unsigned id, const char *descr = NULL);
 
 #ifdef	WIN32
@@ -964,7 +966,15 @@ public:
 				const char *exit;
 				const char *menu;
 				const char *note;
-				const char *list[MAX_LIST];
+				union {
+					const char *list[MAX_LIST];
+					struct {
+						const char *pathv[4];
+						char path1[128];
+						char path2[128];
+						char meta[MAX_LIST * 2];
+					};
+				};
 			} 	audio;
 				
 			struct
