@@ -648,8 +648,12 @@ bool BayonneRPC::buildResponse(const char *fmt, ...)
 		case 'd':
 		case 't':
 		case 'x':
+		case 'm':
 			switch(*fmt)
 			{
+			case 'm':
+				valtype = "string";
+				break;
 			case 'd':
 				valtype = "double";
 				break;
@@ -668,6 +672,57 @@ bool BayonneRPC::buildResponse(const char *fmt, ...)
 			case 'x':
 				valtype = "base64";
 			}
+			if(struct_flag && *fmt == 'm')
+			{
+				if(count > max - 60)
+					goto skip;
+				sv = va_arg(args, const char *);
+				while(sv && *sv)
+				{
+					count += xmlformat(buffer + count, max - count,
+						"  <member><name>");
+					while(*sv && *sv != ':' && *sv != '=' && count < max - 35)
+					{
+						buffer[count++] = tolower(*sv);
+						++sv;
+					}
+					buffer[count] = 0;
+					count += xmlformat(buffer + count, max - count, 
+						"</name>\r\n"
+						"   <value><%s>", valtype);
+					if(*sv == ':' || *sv == '=')
+						++sv;
+					else
+						sv="";
+					while(*sv && *sv != ';' && count < max - 20)
+					{
+						switch(*sv)
+						{
+						case '<':
+							count += xmlformat(buffer + count, max - count, "&lt;");
+							break;
+						case '>':
+							count += xmlformat(buffer + count, max - count, "&gt;");
+							break;
+						case '&':
+							count += xmlformat(buffer + count, max - count, "&amp;");
+							break;
+						case '\"':
+							count += xmlformat(buffer + count, max - count, "&quot;");
+							break;
+						case '\'':
+							count += xmlformat(buffer + count, max - count, "&apos;");
+							break;
+						default:
+							buffer[count++] = *sv;
+						}
+						++sv;
+					}
+					count += xmlformat(buffer + count, max - count,
+						"</%s></value></member>\r\n", valtype);
+				}
+				goto skip;
+			}
 			if(struct_flag)
 			{
 				name = va_arg(args, const char *);
@@ -720,7 +775,7 @@ bool BayonneRPC::buildResponse(const char *fmt, ...)
 			else
 				count += xmlformat(buffer + count, max - count,
 					"</%s></value>\r\n", valtype);
-
+skip:
 			if(!struct_flag && !array_flag)
 				end_flag = true;
 		}
