@@ -51,6 +51,7 @@ ScriptInterp(), audio()
 	peer = NULL;
 	connecting = false;
 	ring = NULL;
+	vm = NULL;
 
 	if(!translator)
 		translator = &langNone;
@@ -102,6 +103,22 @@ BayonneSession::~BayonneSession()
 	finalize();
 }
 
+bool BayonneSession::signalScript(signal_t signal)
+{
+	if(vm)
+		if(vm->signalEngine(signal))
+			return true;
+
+	if(ScriptInterp::signal(signal))
+	{
+		if(vm)
+			vm->releaseEngine();
+		return true;
+	}
+
+	return false;
+}
+
 void BayonneSession::enterCall(void)
 {
 	BayonneService *svc = BayonneService::first;
@@ -316,7 +333,6 @@ void BayonneSession::associate(BayonneSession *s)
 
 	time(&time_joined);
 	strcpy(var_pid, s->var_sid);
-	enterCall();
 }
 
 bool BayonneSession::isAssociated(void)
@@ -1493,6 +1509,7 @@ void BayonneSession::makeIdle(void)
 	strcpy(var_joined, "none");
 	strcpy(var_pid, "none");
 	starting = false;
+	vm = NULL;
 }
 
 timeout_t BayonneSession::getTimeoutValue(const char *opt)
