@@ -1362,7 +1362,7 @@ bool BayonneSession::stateJoin(Event *event)
 	case AUDIO_IDLE:
 		clrAudio();
 		return true;
-        case ENTER_STATE:
+    case ENTER_STATE:
 		if(!strcmp(var_recall, var_pid) && time_joined)
 			exitCall("release");
 		p = getSid(var_recall);
@@ -2295,6 +2295,12 @@ bool BayonneSession::stateRunning(Event *event)
 		if(holding)
 			return true;
 		clrAudio();
+		if(vm)
+		{
+			if(vm->stepEngine())
+					startTimer(step_timer);
+			return true;
+		}
 		step();
 		check();
 		if(state.handler == &BayonneSession::stateRunning)
@@ -2310,6 +2316,9 @@ bool BayonneSession::stateRunning(Event *event)
 		if(holding)
 			startTimer(step_timer);
 		return enterCommon(event);
+	case EXIT_SCRIPT:
+		startTimer(step_timer);
+		return enterCommon(event);
 	case CALL_NOHOLD:
 		if(!holding)
 			return false;
@@ -2350,6 +2359,8 @@ bool BayonneSession::stateHangup(Event *event)
 
 	if(event->id == ENTER_STATE)
 	{
+		if(vm)
+			vm->disconnectEngine();
 		decActiveCalls();
 		holding = false;
 	}
@@ -2380,6 +2391,11 @@ bool BayonneSession::stateHangup(Event *event)
 
 	case LINE_ON_HOOK:
 	case TIMER_EXPIRED:
+		if(vm)
+		{
+			startTimer(100);
+			return true;
+		}
 		clrAudio();
 		setState(STATE_IDLE);
 		return true;
@@ -2514,6 +2530,13 @@ bool BayonneSession::enterCommon(Event *event)
 
 	switch(event->id)
 	{
+	case EXIT_SCRIPT:
+		if(vm)
+		{
+			delete vm;
+			vm = NULL;
+		}
+		return true;
 	case ENABLE_LOGGING:
 	case DISABLE_LOGGING:
 		return true;
