@@ -549,15 +549,16 @@ post:
 
 void BayonneTSession::sysArgs(const char *tsid)
 {
-        char buf[64];
-        const char *cp;
+	char buf[64];
+	const char *cp;
 	char *up;
 	unsigned opt = 0;
 	Line *line = getLine();
+	const char *id, *value;
 
-        enter();
-        if(isLibexec(tsid))
-        {  
+	enter();
+	if(isLibexec(tsid))
+	{  
 		libWrite("300 ARGUMENTS\n");
 		frame[stack].index = 0;
 		if(strnicmp(line->cmd, "exec", 4))
@@ -588,8 +589,32 @@ void BayonneTSession::sysArgs(const char *tsid)
 				libWrite("\n");
 			}
 		}
+		while(line->next && !stricmp(line->next->cmd, "param"))
+		{
+			skip();
+			line = getLine();
+			opt = 0;
+			while(opt < line->argc)
+			{
+				id = line->args[opt++];
+				if(*id != '=')
+					continue;
+				++id;
+				value = getContent(line->args[opt++]);
+				if(!*id || !value)
+					continue;
+				snprintf(buf, sizeof(buf), "PARAM_%s: %s\n", id, value);
+				up = buf;
+				while(*up && *up != ':')
+				{
+					*up = toupper(*up);
+					++up;
+				}
+				libWrite(buf);
+			}
+		}
 		libWrite("\n");
-	}
+	}	
 	leave();
 }
 
@@ -820,7 +845,6 @@ void BayonneTSession::sysHeader(const char *tsid)
 		snprintf(buf, sizeof(buf), "CREF: %s\n", cp);
 		libWrite(buf);
 		libWrite("\n");
-
 	}
 	leave();
 }	
