@@ -1596,7 +1596,7 @@ const char *Session::checkAudio(bool live)
 	switch(info.encoding)
 	{
 	case g721ADPCM:
-		audio.libext = ".adpcm";
+		audio.libext = ".a32";
 		break;
 	case alawAudio:
 		audio.libext = ".al";
