@@ -261,6 +261,27 @@ void Session::run(void)
 	unsigned long xmlsize = 0;
 	bool xmltext = false;
 	BayonneDriver *d;
+	unsigned pos = 0;
+
+	if(!isPending(pendingInput, 10000))
+		goto timeout;
+
+	if(::recv(so, agent, 8, MSG_PEEK) < 8)
+	{
+failed:
+        slog.notice("webservice/%d: connection invalid", so);		
+		syncExit();
+	}	
+
+	while(pos < 8)
+	{
+		if(agent[pos] != 0x03 && agent[pos] != 0x01)
+		{
+			++pos;
+			continue;
+		}
+		goto failed;
+	}
 
 	if(readLine(request, sizeof(request), 10000) < 1)
 	{
@@ -594,16 +615,41 @@ events:
 
 void Session::postDocument(const char *file, unsigned long size)
 {
-    char buf[256];
+    char nbuf[256];
+	char tbuf[256];
 	char dbuf[512];
-    FILE *fp;
+    FILE *fp = NULL;
 	int len;
+	char *ep;
+	const char *ext = strrchr(file, '.');
+
+	if(strstr(file, ".."))
+		goto nowrite;
+
+	if(strstr(file, "/."))
+		goto nowrite;
+
+	if(!strchr(file, '/'))
+		goto nowrite;
+
+	if(!ext || ext < strchr(file, '/'))
+		goto nowrite;
+
+	if(strchr(file, ':'))
+		goto nowrite;
 	
-	snprintf(buf, sizeof(buf), "%s/%s",
+	snprintf(nbuf, sizeof(nbuf), "%s/%s",
 		Service::webservice.getString("datafiles"), file);
 
-	fp = fopen(buf, "w");
+	strcpy(tbuf, nbuf);
+	ep = strrchr(tbuf, '/') + 1;
+	*ep = 0;
+	snprintf(ep, 256 - strlen(tbuf), ".tmp.%d.%s", so, ext);
+
+	remove(tbuf);
+	fp = fopen(tbuf, "w");
 	if(!fp)
+nowrite:
 		rpcFault(403, "Not Writable");
 
 	while(size)
@@ -616,7 +662,7 @@ void Session::postDocument(const char *file, unsigned long size)
 		if(readData(dbuf, len) < len)
 		{
 fault:
-			remove(buf);
+			remove(tbuf);
 			fclose(fp);
 			rpcFault(403, "Write Failed");
 		}
@@ -624,6 +670,7 @@ fault:
 			goto fault;
 	}
 	fclose(fp);
+	rename(tbuf, nbuf);
 	rpcReply(NULL);
 }
 
