@@ -60,7 +60,7 @@ static void server_version(BayonneRPC *rpc)
         return;
     }
 	rpc->buildResponse("(ii)", 
-		"current", (rpcint_t)2,
+		"current", (rpcint_t)3,
 		"prior", (rpcint_t)0);
 }
 
@@ -132,6 +132,68 @@ static void server_callstats(BayonneRPC *rpc)
 		);
 }
 
+static void server_isScript(BayonneRPC *rpc)
+{
+	ScriptImage *img = Bayonne::useImage();
+	Script::Name *scr;
+	const char *id;
+	bool rtn = false;
+
+	if(rpc->getCount() != 1)
+	{
+		rpc->sendFault(3, "Invalid Parameters");
+		goto done;
+	}
+
+	id = rpc->getIndexed(1);
+	if(!id || !*id)
+	{
+		rpc->sendFault(4, "Script Missing");
+		goto done;
+	}
+
+	scr = img->getScript(id);
+	if(scr && scr->access == Script::scrPUBLIC)
+		rtn = true;
+
+	rpc->buildResponse("b", rtn);
+
+done:
+	if(img)
+		Bayonne::endImage(img);
+}
+
+static void server_isSelect(BayonneRPC *rpc)
+{
+	ScriptImage *img = Bayonne::useImage();
+	Script::Name *scr;
+	const char *id;
+	bool rtn = false;
+
+	if(rpc->getCount() != 1)
+	{
+		rpc->sendFault(3, "Invalid Parameters");
+		goto done;
+	}
+
+	id = rpc->getIndexed(1);
+	if(!id || !*id)
+	{
+		rpc->sendFault(4, "Script Missing");
+		goto done;
+	}
+
+	scr = img->getScript(id);
+	if(scr && scr->select)
+		rtn = true;
+
+	rpc->buildResponse("b", rtn);
+
+done:
+	if(img)
+		Bayonne::endImage(img);
+}
+
 static void server_listDestinations(BayonneRPC *rpc)
 {
 	const char *list[1024];
@@ -141,7 +203,7 @@ static void server_listDestinations(BayonneRPC *rpc)
 	if(rpc->getCount())
     {
         rpc->sendFault(3, "Invalid Parameters");
-        return;
+        goto done;
     }
 
 	rpc->buildResponse("[");
@@ -150,6 +212,7 @@ static void server_listDestinations(BayonneRPC *rpc)
 	while(pos < count)
 		rpc->buildResponse("!s", list[pos++]);
 	rpc->buildResponse("]");
+done:
 	if(img)
 		Bayonne::endImage(img);
 }
@@ -425,6 +488,10 @@ static Bayonne::RPCDefine dispatch[] = {
 		"Clear global symbol", "boolean, string"},
 	{"createSymbol", server_createSymbol,
 		"Create global symbol of specified size", "boolean, string, int"},
+	{"isScript", server_isScript,
+		"Check if a given script exists", "boolean, string"},
+	{"isSelect", server_isSelect,
+		"Check if a given script is selectable", "boolean, string"},
 	{"isDestination", server_isDestination,
 		"Check if a given destination is configured", "boolean, string"},
 	{"listDestinations", server_listDestinations,
