@@ -51,6 +51,63 @@ const char *Checks::chkParam(Line *line, ScriptImage *img)
 	return NULL;
 }
 
+const char *Checks::chkId(Line *line, ScriptImage *img)
+{
+	const char *cp = getMember(line);
+	unsigned idx = 0;
+
+	if(cp)
+	{
+		if(!stricmp(cp, "contact"))
+			goto use;
+		if(!strnicmp(cp, "host", 4))
+			goto use;
+		if(!stricmp(cp, "uniq"))
+			goto use;
+		if(!strnicmp(cp, "base", 4))
+			goto use;
+		if(!strnicmp(cp, "dtmf", 4))
+			goto use;
+	
+		return "id may use .contact, .base, .uniq, .dtmf, or .host only";
+	}
+use:
+	if(!useKeywords(line, "=prefix=extension=size=id=idprefix"))
+		return "id keyword invalid";
+
+	while(NULL != (cp = getOption(line, &idx)))
+	{
+		if(*cp != '%' && *cp != '&')
+			return "arguments must be symbol";
+	}
+	return NULL;
+}		
+
+const char *Checks::chkDetach(Line *line, ScriptImage *img)
+{
+	unsigned idx = 0;
+
+	if(getMember(line))
+		return "detach has no members";
+
+	if(!getOption(line, &idx))
+		return "detach requires at least command argument";
+
+	return NULL;
+}
+
+const char *Checks::chkReconnect(Line *line, ScriptImage *img)
+{
+	unsigned idx = 0;
+	if(!useKeywords(line, "=encoding=framing"))
+		return "invalid keyword used for reconnect";
+
+	if(getOption(line, &idx))
+		return "no arguments for reconnect";
+
+	return NULL;
+}
+
 const char *Checks::chkExit(Line *line, ScriptImage *img)
 {
 	if(getMember(line))
@@ -196,7 +253,7 @@ const char *Checks::chkList(Line *line, ScriptImage *img)
 	if(getMember(line))
 		return "list has no members";
 
-	if(!useKeywords(line, "=prefix=match=extension=after"))
+	if(!useKeywords(line, "=prefix=match=extension=after=prior"))
 		return "invalid keyword for list";
 
 	cp = getOption(line, &idx);
