@@ -15,11 +15,282 @@
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
 #include "server.h"
+#ifndef	WIN32
+#include <sys/wait.h>
+#endif
 
 namespace server {
 using namespace ost;
 using namespace std;
 
+bool Methods::scrId(void)
+{
+	Symbol *sym;
+	const char *cp = getKeyword("size");
+	const char *m = getMember();
+	time_t now;
+	const char *pre, *host;
+	char *dp;
+	const char *idprefix = getKeyword("idprefix");
+	const char *prefix = getKeyword("prefix");
+	const char *ext = getKeyword("extension");
+	const char *id = getKeyword("id");
+	unsigned size = 0;
+	unsigned short uniq = 0;
+	char buffer[128];
+	unsigned char code;
+
+	if(!idprefix)
+		idprefix = "";
+
+	if(prefix && (*prefix == '.' || *prefix == '/'))
+	{
+		error("invalid-path");
+		goto clearall;
+	}
+
+	if(prefix && (strstr(prefix, "/.") || strchr(prefix, ':')))
+	{
+		error("invalid-path");
+		goto clearall;
+	}
+	
+	if(cp)
+		size = atoi(cp);
+	else if(prefix && id)
+		size = 128;
+	else if(m && !stricmp(m, "uniq") && prefix)
+		size = 128;
+	else if(m && !stricmp(m, "uniq"))
+		size = 16;
+	else if(id)
+		size = strlen(id);
+
+	if(!ext)
+		ext = ".au";
+
+	time(&now);
+	while(NULL != (cp = getOption(NULL)))
+	{
+		if(*cp == '%' || *cp == '&')
+			++cp;
+		sym = mapSymbol(cp, size);
+		if(!sym)
+			continue;
+		if(sym->type != symNORMAL && sym->type != symCONST && sym->type != symINITIAL)
+		{
+			error("cannot-access");
+			return true;
+		}
+		if(prefix && sym->type == symCONST)
+		{
+			error("cannot-access");
+			return true;
+		}
+		if(!sym->data[0] && id)
+			commit(sym, id);
+		if(sym->type == symINITIAL)
+			sym->type = symNORMAL;
+		if(m && !stricmp(m, "uniq"))
+		{
+			if(sym->type == symCONST)
+				continue;
+			snprintf(sym->data, sym->size + 1, "%08lx-%02d%04d", 
+				now, ++uniq, timeslot);				
+			goto modify;
+		}
+		if(m && !stricmp(m, "contact"))
+		{
+			pre = strchr(sym->data, ':');
+			host = strchr(sym->data, '@');
+			dp = sym->data;
+			if(!pre || (host && pre > host))
+				pre = sym->data;
+			else
+				++pre;
+			if(*pre == '+')
+				++pre;
+			while(*pre && *pre != ':')
+				*(dp++) = *(pre++);
+			*dp = 0;
+			goto modify;
+		}	
+		if(m && !strnicmp(m, "base", 4))
+		{
+			pre = strrchr(sym->data, '/');
+			if(!pre)
+				goto modify;			
+			++pre;
+			goto update;
+		}
+		if(m && !strnicmp(m, "host", 4))
+		{
+            host = strchr(sym->data, '@');
+            dp = sym->data;
+            if(!host)
+                host = sym->data;
+            else
+                ++host;
+            while(*host && *host != ':')
+                *(dp++) = *(host++);
+            *dp = 0;
+            goto modify;
+		}				
+		if(m && !stricmp(m, "dtmf"))
+		{
+			dp = sym->data;
+			pre = sym->data;
+			while(*pre)
+			{
+				code = (unsigned char)*(pre++);
+				if(dtmf_keymap[code])
+					*(dp++) = dtmf_keymap[code];
+			}
+			*dp = 0;
+			goto modify;
+		}				
+		pre = getRegistryId(sym->data);
+		if(*pre == '+')
+			++pre;
+update:
+		dp = sym->data;
+		while(pre && *pre && *pre != ':' && *pre != '@')
+			*(dp++) = *(pre++);
+		*dp = 0;
+modify:
+		if(!prefix)
+			continue;
+
+		if(strchr(sym->data, ':') || strchr(sym->data, '/') || sym->data[0] == '.')
+		{
+			clear(sym);
+			continue;
+		}
+		snprintf(buffer, sizeof(buffer), "%s/%s%s%s", 
+			prefix, idprefix, sym->data, ext);
+		commit(sym, buffer);
+	}
+	advance();
+	return true;
+clearall:
+	while(NULL != (cp = getOption(NULL)))
+	{
+		sym = mapSymbol(cp, 0);
+		if(sym)
+			clear(sym);
+	}
+	return true;
+}
+
+#ifdef	WIN32
+bool Methods::scrDetach(void)
+{
+	error("no-detach");
+	return true;
+}
+#else
+bool Methods::scrDetach(void)
+{
+	Line *line = getLine();
+	char *argv[128];
+	int argc = 0;
+	const char *cp = getValue(NULL);
+	const char *v;
+	char *up;
+	int pid, fd;
+	int max_fd = 256;
+	struct rlimit rlim;
+	unsigned idx = 0;
+	char name[65];
+	char execpath[128];
+
+	v = strrchr(cp, '/');
+	if(v)
+	{
+		error("cannot use path");
+		return true;
+	}
+	else
+		v = cp;
+
+	snprintf(execpath, sizeof(execpath), "%s/%s", LIBEXEC_FILES, cp);
+	while(NULL != (cp = getValue(NULL)))
+		argv[argc++] = (char *)cp;
+
+	argv[argc] = NULL;
+
+	pid = fork();
+	if(pid < 0)
+	{
+		error("detach-failed");
+		return true;
+	}
+	if(pid > 0)
+	{
+		waitpid(pid, NULL, 0);
+		setSymbol("_detach_", v, 16);
+		advance();
+		return true;
+	}
+	if(!getrlimit(RLIMIT_NOFILE, &rlim))
+		max_fd = rlim.rlim_max;
+	for(fd = 3; fd < max_fd; ++fd)
+		::close(fd);
+	::signal(SIGQUIT, SIG_DFL);
+	::signal(SIGINT, SIG_DFL);
+	::signal(SIGCHLD, SIG_DFL);
+	fd = ::open("/dev/null", O_RDWR);
+	if(fd > 2)
+	{
+		::dup2(fd, 0);
+		::dup2(fd, 1);
+		::close(fd);
+	}
+	pid = fork();
+	if(pid)
+		::exit(0);
+
+	idx = 0;
+	while(idx < line->argc)
+	{
+		cp = line->args[idx++];
+		if(*cp != '=')
+			continue;
+
+		v = getContent(line->args[idx++]);
+		if(!v)
+			continue;
+	
+		snprintf(name, sizeof(name), "param_%s", ++cp);
+		up = name;
+		while(*up)
+		{
+			*up = toupper(*up);
+			++up;
+		}
+		Process::setEnv(name, v, true);
+	}
+		
+	::execv(execpath, argv);
+	::exit(-1);	
+}
+#endif
+	
+bool Methods::scrReconnect(void)
+{
+	const char *encoding = getKeyword("encoding");
+	timeout_t framing = getTimeoutKeyword("framing");
+
+	if(!framing || framing == TIMEOUT_INF)
+		framing = 0;
+
+	if(setReconnect(encoding, framing))
+		return false;
+
+	error("reconnect-unsupported");
+	return true;
+}
+		
 bool Methods::scrParam(void)
 {
 	Line *line = getLine();
@@ -927,8 +1198,7 @@ bool Methods::scrLink(void)
 
 	if(link(cp, dp))
 	{
-		if(!signalScript(SIGNAL_FAIL))
-			error("link-failed");
+		error("link-failed");
 		return true;
 	}
 
@@ -983,8 +1253,7 @@ bool Methods::scrMove(void)
 
 	if(rename(cp, dp))
 	{
-		if(!signalScript(SIGNAL_FAIL))
-			error("move-failed");
+		error("move-failed");
 		return true;
 	}
 
