@@ -63,16 +63,24 @@ ScriptRuntime()
 			(Check)&Checks::chkPaths},
 		{"path", true, (Method)&Methods::scrPath,
 			(Check)&Checks::chkPaths},
+		{"id", true, (Method)&Methods::scrId,
+			(Check)&Checks::chkId},
+        {"idpath", true, (Method)&Methods::scrId,
+            (Check)&Checks::chkId},
 		{"param", true, (Method)&Methods::scrParam,
 			(Check)&Checks::chkParam},
 		{"voicelib", false, (Method)&Methods::scrVoicelib,
 			(Check)&Checks::chkVoicelib},
 		{"timeslot", true, (Method)&Methods::scrTimeslot,
 			(Check)&ScriptChecks::chkType},
+		{"reconnect", false, (Method)&Methods::scrReconnect,
+			(Check)&Checks::chkReconnect},
+        {"detach", false, (Method)&Methods::scrDetach,
+            (Check)&Checks::chkDetach},
 		{"position", true, (Method)&Methods::scrPosition,
 			(Check)&ScriptChecks::chkType},
-        	{"echo", false, (Method)&Methods::scrEcho,
-                	(Check)&ScriptChecks::chkHasArgs},
+        {"echo", false, (Method)&Methods::scrEcho,
+			(Check)&ScriptChecks::chkHasArgs},
 		{"pickup", false, (Method)&Methods::scrPickup,
 			(Check)&ScriptChecks::chkOnlyCommand},
                 {"hangup", false, (Method)&Methods::scrHangup, 
@@ -190,6 +198,7 @@ ScriptRuntime()
 	load(interp);
 
 	ScriptInterp::addConditional("runlevel", &testLevel);
+	ScriptInterp::addConditional("safe", &testSafe);
 	ScriptInterp::addConditional("file", &testFile);
 	ScriptInterp::addConditional("dir", &testDir);
 	ScriptInterp::addConditional("span", &testSpan);
@@ -201,6 +210,7 @@ ScriptRuntime()
 	ScriptInterp::addConditional("pattern", &testPattern);
 	ScriptInterp::addConditional("dialed", &testDialed);
     ScriptInterp::addConditional("caller", &testCaller);
+    ScriptInterp::addConditional("detach", &testDetach);
 }
 
 unsigned long Runtime::getTrapMask(const char *trapname)
@@ -340,6 +350,31 @@ bool Runtime::testDriver(ScriptInterp *interp, const char *v)
 	return false;
 }
 
+bool Runtime::testDetach(ScriptInterp *interp, const char *v)
+{
+	const char *dj = interp->getSymbol("_detach_");
+	if(!dj || !v)
+		return false;
+
+	if(!stricmp(dj, v))
+		return true;
+	return false;
+}
+
+bool Runtime::testSafe(ScriptInterp *interp, const char *v)
+{
+	if(*v == '.' || *v == '/')
+		return false;
+
+	if(strchr(v, ':'))
+		return false;
+
+	if(strstr(v, "/."))
+		return false;
+
+	return true;
+}
+
 bool Runtime::testFile(ScriptInterp *interp, const char *v)
 {
 	BayonneSession *s = (BayonneSession *)(interp);
